#include <Wire.h>
#include "TimerOne.h"  
#include <InternalTemperature.h>
#include <EEPROM.h>

InternalTemperature temperature;
unsigned long lastDebounceTime = 0; 
static const unsigned long REFRESH_INTERVAL = 5; // ms
static unsigned long lastRefreshTime = 0;

double measuredFrequency=0;
int pulsecount=0;

String cmd="";
void loop(){
  cmd="";
  while (Serial.available() > 0)
  {
    cmd += (char) Serial.read(); // Read in one char at a time
    delay(5); // Delay for 5 ms so the next char has time to be received
  }
  if (cmd!=""){
      parseCommand();
    }

  //measuredFrequeny calculation
  long trueinterval=0;
  
  trueinterval = (millis() - lastRefreshTime)/1000.;
  if(trueinterval >= REFRESH_INTERVAL){
    measuredFrequency=1.0*pulsecount/trueinterval;
    pulsecount=0;
    lastRefreshTime=millis();
  }
  StatusLEDs();
}

//state variables followed by their EEPROM Address
double frequency=1; // 4 byte
int frequencyAddr=10;

double biasV=0; // 4 byte
int biasVAddr=20;

boolean enabled=false;

int triggerMode=0; //internal //2 byte
int triggerModeAddr=30;

int ledMode=1; //on // 2 byte
int ledModeAddr=40;

void setBias() {
  int vbit=int((biasV+2.425)/3.3 *4095);
  analogWrite(A12,vbit); // 12bit DAC at pin 12
}

void setup() {
  Serial.begin(9600);
  
  pinMode(2,OUTPUT); //TRIGGER LED
  pinMode(3,OUTPUT); //ENABLED LED
  pinMode(4,OUTPUT); //POWER LED
  pinMode(23,OUTPUT); //SWITCH
  pinMode(22,INPUT_PULLUP); // TRIGGER IN
  pinMode(21,OUTPUT); //PULSE
  digitalWrite(21,LOW);
  digitalWrite(23,LOW); //start turned off

  temperature.begin();
  analogWriteResolution(12);

  //read EEPROM

  //check if EEPROM has been set

  if(EEPROM.read(0)!=22){
    EEPROM.put(frequencyAddr, 1); delay(10);
    EEPROM.put(biasVAddr, 0); delay(10);
    EEPROM.put(triggerModeAddr, 0); delay(10);
    EEPROM.put(ledModeAddr, 1); delay(10);
    EEPROM.write(0,22); delay(10);
   }else{
    EEPROM.get(frequencyAddr, frequency); delay(10);
    EEPROM.get(biasVAddr, biasV); delay(10);
    EEPROM.get(triggerModeAddr, triggerMode); delay(10);
    EEPROM.get(ledModeAddr, ledMode); delay(10);
  }
  
  setBias();

  Timer1.initialize(1000000/frequency);  //set frequency
  
  if(triggerMode==1){
     Timer1.detachInterrupt();
     attachInterrupt(22, fCounter, FALLING);
  }else{
     Timer1.attachInterrupt(pulse);
     detachInterrupt(digitalPinToInterrupt(22));
  } 
  //

}

void parseCommand() {
  cmd=cmd.substring(0,cmd.length() - 2);

  if(cmd.equals("help")) {
     Serial.println("");
     Serial.println("Available commands:");
     Serial.println("type: 'status' to get configuration and chip temperature");
     Serial.println("type: 'frequency xx' to specify the LED pulse frequency in Hertz (max. 25kHz)");
     Serial.println("type: 'bias xx' to specify the offset voltage in V (-2.4V to +0.7V)");
     Serial.println("type: LEDs: 'on' or 'off' to enable or disable the status LEDs");
     Serial.println("type: trigger: 'internal' or 'external'");
     Serial.println("type: 'on' to enable pulsed light output");
     Serial.println("type: 'off' to disable pulsed light output");
     Serial.println("type: 'help' for these instructions");
     Serial.println("------------------------------------");
  }else if(cmd.equals("status")) {
    getStatus();
  }else if(cmd.indexOf("frequency")!=-1) {
    String val=cmd.substring(10,cmd.length());
    frequency=val.toFloat();
    EEPROM.put(frequencyAddr, frequency);
    if (frequency>0){
      if (frequency<=25000){
        Timer1.initialize(1000000/frequency);
      }else{
        Serial.println("Maximum frequency is 25kHz.");
      }
    }else{
      Serial.println("Frequency must be positive.");
    }
  }else if(cmd.indexOf("bias")!=-1) {
    String val=cmd.substring(5,cmd.length());
    if ((val.toFloat()<=0.7) && (val.toFloat()>=-2.41)){
      biasV=val.toFloat();
      EEPROM.put(biasVAddr, biasV);
      setBias();
    }else{
      Serial.println("Requested offset outside range.");
    }
  }else if(cmd.equals("on")) {
    digitalWrite(23,HIGH);
    enabled=true;
  }else if(cmd.equals("off")) {
    digitalWrite(23,LOW);
    enabled=false;
  }else if(cmd.indexOf("trigger")!=-1) {
    if(cmd.indexOf("internal")!=-1){
        triggerMode=0;
        EEPROM.put(triggerModeAddr, triggerMode);
        Timer1.attachInterrupt(pulse);
        detachInterrupt(digitalPinToInterrupt(22));
    }else if(cmd.indexOf("external")!=-1){ 
        triggerMode=1;
        EEPROM.put(triggerModeAddr, triggerMode);
        Timer1.detachInterrupt();
        attachInterrupt(22, fCounter, FALLING);
    }
  }else if(cmd.indexOf("LEDs")!=-1) {
    if(cmd.indexOf("on")!=-1){
      ledMode=1;
      EEPROM.put(ledModeAddr, ledMode);
    }else if(cmd.indexOf("off")!=-1){ 
      ledMode=0;
      EEPROM.put(ledModeAddr, ledMode);
    }
  }
  cmd = "";
}

void fCounter(){
  pulsecount=pulsecount+1;
  if(ledMode==1){
    digitalWrite(2,HIGH);
    lastDebounceTime=millis();
  }
}

void StatusLEDs(){
  if ((millis()-lastDebounceTime)>10){
    digitalWrite(2,LOW);
  }
  if(ledMode==0){
    digitalWrite(2,LOW);
    digitalWrite(3,LOW);
    digitalWrite(4,LOW);
  }else{
    //2 Trigger, 3 Enable
    digitalWrite(4,HIGH); //POWER
    if (enabled){
      digitalWrite(3,HIGH);
    }else{
      digitalWrite(3,LOW);
    }
  }
}

void pulse(){
  digitalWriteFast(21,HIGH);
  digitalWriteFast(21,LOW);
  if(ledMode==1){
    digitalWrite(2,HIGH);
    lastDebounceTime=millis();
  }
}
               
void getStatus(){
  if(enabled){
      Serial.println("LED OUTPUT ENABLED!");
  
  }else{
      Serial.println("LED OUTPUT DISABLED!");  
  }
  Serial.print("Trigger mode: ");
  if (triggerMode==0){
    Serial.println("internal");
    Serial.print("Frequency (set) [Hz]: ");
    Serial.println(frequency);
  }else{
    Serial.println("external");
    Serial.print("Frequency (measured) [Hz]: ");
    Serial.println(measuredFrequency);
  }
  Serial.print("Set bias voltage [V]: ");
  Serial.println(biasV);
  Serial.print("Bias voltage readout [V]: ");
  Serial.println(2*analogRead(A0)/65536.*3.3 -3.3);
  Serial.print("MC temperature[C]: ");
  Serial.println(temperature.readTemperatureC());
  Serial.println("------------------------------------");
}


